//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Util/Assert.h
//! @brief     Defines the macro ASSERT.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_BASE_UTIL_ASSERT_H
#define BORNAGAIN_BASE_UTIL_ASSERT_H

#include "Wrap/WinDllMacros.h"
#include <functional>
#include <string>

// Function called upon failed assert; set in Assert.cpp, overriden by GUI.
extern BA_BASE_API_ std::function<void(std::string)> failedAssertion;

// ASSERT macro: terminate if condition is false.
//
// Implementation notes:
// - Must be declared as a macro, not a function, so that we can use preprocessor
//   macros for informative error messages.
// - Must terminate with a throw statement to  prevent compiler warning -Wreturn-type.
// - In the GUI, the function pointer failedAssertion will be reset to call qFatal,
//   which then pops up a message window (as defined by qInstallMessageHandler).
// - No direct call to qFatal here, because we do not want core to depend on Qt,
//   lest Python wheel becomes too difficult.

#ifdef BA_DEBUG

#include <csignal>
#include <iostream>
#define ASSERT(condition)                                                                          \
    if (!(condition)) {                                                                            \
        std::cerr << "Assertion " << (#condition) << " failed in " << __FILE__ << ", line "        \
                  << __LINE__ << std::endl;                                                        \
        std::raise(SIGTERM); /* abort so that we can inspect the backtrace */                      \
        throw std::runtime_error("Assertion failed ... and we should never get here");             \
    }

#else

#include <sstream>
#include <stdexcept>
#define ASSERT(condition)                                                                          \
    if (!(condition)) {                                                                            \
        std::stringstream msg;                                                                     \
        msg << "Assertion " << (#condition) << " failed in " << __FILE__ << ", line " << __LINE__; \
        failedAssertion(msg.str());                                                                \
        throw std::runtime_error("Assertion failed ... and we should never get here");             \
    }

#endif // BA_DEBUG

#endif // BORNAGAIN_BASE_UTIL_ASSERT_H
